'use strict';

var _get = require('babel-runtime/helpers/get')['default'];

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
    value: true
});

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _allureJsCommons = require('allure-js-commons');

var _allureJsCommons2 = _interopRequireDefault(_allureJsCommons);

var _allureJsCommonsBeansStep = require('allure-js-commons/beans/step');

var _allureJsCommonsBeansStep2 = _interopRequireDefault(_allureJsCommonsBeansStep);

function isEmpty(object) {
    return !object || _Object$keys(object).length === 0;
}

var LOGGING_HOOKS = ['"before all" hook', '"after all" hook'];

var debug = false;
var debugSeleniumCommand = false;

function logger() {
    if (debug) {
        console.log(arguments);
    }
}
function error() {
    console.error(arguments);
}

/**
 * Initialize a new `Allure` test reporter.
 *
 * @param {Runner} runner
 * @api public
 */

var AllureReporter = (function (_events$EventEmitter) {
    _inherits(AllureReporter, _events$EventEmitter);

    function AllureReporter(baseReporter, config) {
        var _this2 = this;

        var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

        _classCallCheck(this, AllureReporter);

        _get(Object.getPrototypeOf(AllureReporter.prototype), 'constructor', this).call(this);

        this.baseReporter = baseReporter;
        this.config = config;
        this.options = options;
        if (options) {
            debug = options.debug;
            debugSeleniumCommand = options.debugSeleniumCommand;
        }
        this.allures = {};
        this.failedSpecs = {};
        this.verifiedSpecs = {};

        var epilogue = this.baseReporter.epilogue;

        this.on('end', function () {
            epilogue.call(baseReporter);
        });

        this.on('suite:start', function (suite) {
            var allure = _this2.getAllure(suite.cid);
            var currentSuite = allure.getCurrentSuite();
            var prefix = currentSuite ? currentSuite.name + ' ' : '';

            process.workflo.currentCid = suite.cid;

            allure.startSuite(prefix + suite.title);
        });

        this.on('suite:end', function (suite) {
            _this2.getAllure(suite.cid).endSuite();
        });

        this.on('test:start', function (test) {
            var allure = _this2.getAllure(test.cid);
            allure.startCase(test.title);

            var currentTest = allure.getCurrentTest();
            currentTest.addParameter('environment-variable', 'capabilities', JSON.stringify(test.runner[test.cid]));
            currentTest.addParameter('environment-variable', 'spec files', JSON.stringify(test.specs));

            allure.workflo.failedSpecs = [];
            allure.workflo.pendingSpecCriterias = [];
            allure.workflo.pendingSpec = undefined;
        });

        this.on('test:pass', function (test) {
            var allure = _this2.getAllure(test.cid);
            var status = 'passed';
            var err = undefined;

            if (allure.workflo.specMode) {
                if (allure.workflo.failedSpecs.length > 0) {
                    status = 'failed';

                    var newErr = {
                        message: '',
                        stack: ''
                    };

                    newErr.message = allure.workflo.failedSpecs.map(function (err) {
                        return err.message;
                    }).reduce(function (prev, cur) {
                        return prev + '\n' + cur;
                    });

                    newErr.stack = allure.workflo.failedSpecs.map(function (err) {
                        return err.stack;
                    }).reduce(function (prev, cur) {
                        return prev + '\n\n' + cur;
                    });

                    newErr.message = '\n' + newErr.message + '\n';
                    newErr.stack = '\n' + newErr.stack;

                    err = newErr;
                } else if (allure.workflo.pendingSpecCriterias.length > 0) {
                    status = 'failed';
                    err = {
                        message: 'Spec ' + allure.workflo.pendingSpec + ': ' + allure.workflo.pendingSpecCriterias.toString() + ' was not verified!',
                        stack: ''
                    };
                    //this.getAllure(test.cid).pendingCase(test.title)
                }
            }

            allure.endCase(status, err);
        });

        this.on('test:fail', function (test) {
            var allure = _this2.getAllure(test.cid);
            var failedSpecs = _this2.getFailedSpecs();
            //const status = test.err.type === 'AssertionError' ? 'failed' : 'broken'
            var status = test.err.matcherName !== '' ? 'failed' : 'broken';

            if (!allure.getCurrentTest()) {
                allure.startCase(test.title);
            } else {
                allure.getCurrentTest().name = test.title;
            }

            while (allure.getCurrentSuite().currentStep instanceof _allureJsCommonsBeansStep2['default']) {
                allure.endStep(status);
            }

            var newErr = {
                message: '',
                stack: ''
            };

            if (test.errs && test.errs.length > 0) {

                if (!allure.workflo.specMode) {
                    test.errs.forEach(function (err) {
                        //console.log("ERROR: ", err)

                        for (var key in err.specObj) {
                            if (!(key in failedSpecs)) {
                                failedSpecs[key] = {};
                            }

                            var criterias = err.specObj[key];

                            if (criterias.constructor !== Array) {
                                criterias = [criterias[key]];
                            }

                            var splitIdx = test.parent.lastIndexOf("suite");
                            var suite = test.parent.substring(0, splitIdx);

                            var _iteratorNormalCompletion = true;
                            var _didIteratorError = false;
                            var _iteratorError = undefined;

                            try {
                                for (var _iterator = _getIterator(criterias), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                                    var criteria = _step.value;

                                    if (!(criteria in failedSpecs[key])) {
                                        failedSpecs[key][criteria] = [];
                                    }

                                    failedSpecs[key][criteria].push({
                                        message: err.message,
                                        stack: err.stack,
                                        passed: err.passed,
                                        expected: err.expected,
                                        actual: err.actual,
                                        origin: suite + ': ' + test.title
                                    });
                                }
                            } catch (err) {
                                _didIteratorError = true;
                                _iteratorError = err;
                            } finally {
                                try {
                                    if (!_iteratorNormalCompletion && _iterator['return']) {
                                        _iterator['return']();
                                    }
                                } finally {
                                    if (_didIteratorError) {
                                        throw _iteratorError;
                                    }
                                }
                            }
                        }
                    });

                    newErr.message = test.errs.map(function (err) {
                        return err.message;
                    }).reduce(function (prev, cur) {
                        return prev + '\n' + cur;
                    });

                    newErr.stack = test.errs.map(function (err) {
                        return err.stack;
                    }).reduce(function (prev, cur) {
                        return prev + '\n\n' + cur;
                    });
                }
            } else {
                newErr.message = test.err.message;
                newErr.stack = test.err.stack;
            }

            newErr.message = '\n' + newErr.message + '\n';
            newErr.stack = '\n' + newErr.stack;

            allure.endCase(status, newErr);
        });

        this.on('test:pending', function (test) {
            _this2.getAllure(test.cid).pendingCase(test.title);
        });

        this.on('runner:command', function (command) {
            var allure = _this2.getAllure(command.cid);

            if (!_this2.isAnyTestRunning(allure)) {
                return;
            }

            if (debugSeleniumCommand) {
                allure.startStep(command.method + ' ' + command.uri.path);

                if (!isEmpty(command.data)) {
                    _this2.dumpJSON(allure, 'Request', command.data);
                }
            }
        });

        this.on('runner:result', function (command) {
            var allure = _this2.getAllure(command.cid);
            var status = 'passed';

            if (!_this2.isAnyTestRunning(allure)) {
                return;
            }

            if (debugSeleniumCommand) {
                if (command.requestOptions.uri.path.match(/\/wd\/hub\/session\/[^/]*\/screenshot/)) {
                    allure.addAttachment('Screenshot', new Buffer(command.body.value, 'base64'));
                } else {
                    if (command.body.screenshot) {
                        allure.addAttachment('Screenshot', new Buffer(command.body.screenshot, 'base64'));
                        delete command.body.screenshot;
                    }
                    if (command.body.type && command.body.type === 'RuntimeError') {
                        status = 'broken';
                    }

                    _this2.dumpJSON(allure, 'Response', command.body);
                }
            }

            allure.endStep(status);
        });

        /*
           meta : {
            (mandatory) cid : ...,
            (mandatory) event : 'test:meta'
            (optional) description : string
            (optional) feature : string | array
            (optional) strory : string | array
            (optional) issue : string | array
            (optional) severity : [ 'blocker','critical','normal','minor','trivial' ]
            (optional) argument : {name: value, name2: value }
            (optional) environment : {name: value, name2: value }
           }
        */
        this.on('test:meta', function (meta) {
            var allure = this.getAllure(meta.cid);
            logger('test:meta', meta);

            if (!this.isAnyTestRunning(allure)) {
                error('ERROR', 'test:meta : NO TEST RUNNING');
                return;
            }
            var currentTest = allure.getCurrentTest();

            // manage description
            if (meta.description) {
                currentTest.setDescription(meta.description);
            }
            // manage labels ( feature, story, issue )
            if (meta.feature) {
                if (typeof meta.feature == 'string') {
                    currentTest.addLabel('feature', meta.feature);
                } else {
                    for (var i in meta.feature) {
                        currentTest.addLabel('feature', meta.feature[i]);
                    }
                }
            }
            if (meta.story) {
                if (typeof meta.story == 'string') {
                    currentTest.addLabel('story', meta.story);
                } else {
                    for (var i in meta.story) {
                        currentTest.addLabel('story', meta.story[i]);
                    }
                }
            }
            if (meta.issue) {
                if (typeof meta.issue == 'string') {
                    currentTest.addLabel('issue', meta.issue);
                } else {
                    for (var i in meta.issue) {
                        currentTest.addLabel('issue', meta.issue[i]);
                    }
                }
            }
            if (meta.severity) {
                if (typeof meta.severity == 'string') {
                    currentTest.addLabel('severity', meta.severity);
                } else {
                    error('ERROR : meta.severity should be a string', meta);
                }
            }
            // manage parameters
            if (meta.argument) {
                if (typeof meta.argument == 'object') {
                    var keys = _Object$keys(meta.argument);
                    for (var i in keys) {
                        var key = keys[i];
                        var val = meta.argument[key];
                        currentTest.addParameter('argument', key, val);
                    }
                } else {
                    error('ERROR : meta.argument should be an object { name2: val1, name2: val2.. }', meta);
                }
            }
            if (meta.environment) {
                if (typeof meta.environment == 'object') {
                    var keys = _Object$keys(meta.environment);
                    for (var i in keys) {
                        var key = keys[i];
                        var val = meta.environment[key];
                        currentTest.addParameter('environment-variable', key, val);
                    }
                } else {
                    error('ERROR : meta.environment should be an object { name2: val1, name2: val2.. }', meta);
                }
            }
        });

        this.on('step:failed', function (step) {
            var allure = this.getAllure(step.cid);
            allure.workflo.stepStatus = 'failed';

            var failedAssertion = {
                matcherName: step.assertion.matcherName,
                message: step.assertion.message
            };

            allure.workflo.failedAssertions.push(failedAssertion);

            this.addVerifiedSpecs(step.assertion.specObj);
        });

        this.on('step:succeeded', function (step) {
            var allure = this.getAllure(step.cid);

            this.addVerifiedSpecs(step.assertion.specObj);
        });

        this.on('step:triggerSpecMode', function (step) {
            var allure = this.getAllure(step.cid);
            allure.workflo.specMode = true;
        });

        this.on('step:verifyStart', function (step) {
            var allure = this.getAllure(step.cid);
            allure.workflo.verifyContainer = step.verifyContainer;
        });

        this.on('step:verifyEnd', function (step) {
            var allure = this.getAllure(step.cid);
            allure.workflo.verifyContainer = undefined;
        });

        this.on('step:start', function (step) {
            var allure = this.getAllure(step.cid);
            logger('step:start', step);

            if (!this.isAnyTestRunning(allure)) {
                error('ERROR', 'cannot start step because no test is running', step);
                return;
            }
            allure.startStep(step.title != null ? step.title : 'No name defined');

            if (step.arg) {
                this.dumpJSON(allure, 'Step Arg', step.arg);
            }

            allure.workflo.stepStatus = 'passed';
            allure.workflo.failedAssertions = [];
        });

        this.on('step:end', function (step) {
            var allure = this.getAllure(step.cid);
            var failedSpecs = this.getFailedSpecs();
            var verifiedSpecs = this.getVerifiedSpecs();
            logger('step:end', step);

            if (!this.isAnyTestRunning(allure)) {
                error('ERROR', 'cannot end step because no test is running', step);
                return;
            }

            var status = allure.workflo.stepStatus;

            if (step.verify) {
                if (this.didSpecFail(step.verify.storyId, step.verify.criteriaId)) {
                    status = 'failed';

                    allure.workflo.failedSpecs = allure.workflo.failedSpecs.concat(failedSpecs[step.verify.storyId][step.verify.criteriaId]);

                    this.dumpJSON(allure, 'Verification Failures', failedSpecs[step.verify.storyId][step.verify.criteriaId].map(function (failedAssertion) {
                        return {
                            origin: failedAssertion.origin,
                            message: failedAssertion.message,
                            stack: failedAssertion.stack
                        };
                    }));
                } else if (!this.wasSpecVerified(step.verify.storyId, step.verify.criteriaId)) {
                    allure.workflo.pendingSpecCriterias.push(step.verify.criteriaId);
                    allure.workflo.pendingSpec = step.verify.storyId;
                    status = 'failed';
                }
            }

            if (step.arg) {
                this.dumpJSON(allure, 'Step Result', step.arg);
            }

            if (step.type && step.type === 'verifyEnd' && allure.workflo.stepStatus === 'failed' && allure.workflo.failedAssertions.length > 0) {
                this.dumpJSON(allure, 'Verification Failures', allure.workflo.failedAssertions);
                allure.workflo.verifyContainer = undefined;
            }

            allure.endStep(step.status != null ? step.status : status);
        });
        /*
          attachment : {
            cid: ...,
            event: 'test:attach',
            title: string,
            file: string,
            type: string
          }
        
        */
        this.on('test:attach', function (attachment) {
            var allure = _this.getAllure(attachment.cid);
            logger('test:attach', attachment);

            if (this.isAnyTestRunning(allure)) {
                error('ERROR', 'cannot attach because no test is running', attachment);
                return;
            }
            allure.addAttachment(attachment.title, fs.readFileSync(attachment.file), attachment.type);
        });

        /*
          log : {
            cid: ...,
            event: 'test:log',
            message: string,
            detail: object
          }
        
        */
        this.on('test:log', function (log) {
            var allure = this.getAllure(log.cid);
            logger('test:log', log);

            if (this.isAnyTestRunning(allure)) {
                error('ERROR', 'cannot log because no test is running', log);
                return;
            }
            var content = log.detail != null ? JSON.stringify(log.detail, null, '    ') : '';
            allure.addAttachment(log.message, content, 'application/json');
        });

        this.on('runner:screenshot', function (command) {
            var allure = this.getAllure(command.cid);
            allure.addAttachment('screenshot ' + command.filename, new Buffer(command.data, 'base64'));
        });

        this.on('hook:start', function (hook) {
            var allure = _this2.getAllure(hook.cid);

            if (!allure.getCurrentSuite() || LOGGING_HOOKS.indexOf(hook.title) === -1) {
                return;
            }

            allure.startCase(hook.title);
        });

        this.on('hook:end', function (hook) {
            var allure = _this2.getAllure(hook.cid);

            if (!allure.getCurrentSuite() || LOGGING_HOOKS.indexOf(hook.title) === -1) {
                return;
            }

            allure.endCase('passed');

            if (allure.getCurrentTest().steps.length === 0) {
                allure.getCurrentSuite().testcases.pop();
            }
        });
    }

    _createClass(AllureReporter, [{
        key: 'getAllure',
        value: function getAllure(cid) {
            if (typeof cid === 'undefined') {
                cid = process.workflo.cid;
            } else {
                if (typeof process.workflo === 'undefined') {
                    process.workflo = {};
                } else {
                    process.workflo.currentCid = cid;
                }
            }

            if (this.allures[cid]) {
                return this.allures[cid];
            }

            var allure = new _allureJsCommons2['default']();
            allure.setOptions({ targetDir: this.options.outputDir || 'allure-results' });
            allure.workflo = {
                stepStatus: 'passed',
                verifyContainer: undefined,
                specMode: false, //first testcases are executed -> flase, then specs -> true
                failedSpecs: [],
                pendingSpecCriterias: [],
                pendingSpec: undefined
            };
            this.allures[cid] = allure;
            return this.allures[cid];
        }
    }, {
        key: 'getFailedSpecs',
        value: function getFailedSpecs() {
            return this.failedSpecs;
        }
    }, {
        key: 'getVerifiedSpecs',
        value: function getVerifiedSpecs() {
            return this.verifiedSpecs;
        }
    }, {
        key: 'addVerifiedSpecs',
        value: function addVerifiedSpecs(specObj) {
            var verifiedSpecs = this.verifiedSpecs;

            for (var key in specObj) {
                if (!(key in verifiedSpecs)) {
                    verifiedSpecs[key] = {};
                }

                var criterias = specObj[key];

                if (criterias.constructor !== Array) {
                    criterias = [criterias[key]];
                }

                var _iteratorNormalCompletion2 = true;
                var _didIteratorError2 = false;
                var _iteratorError2 = undefined;

                try {
                    for (var _iterator2 = _getIterator(criterias), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                        var criteria = _step2.value;

                        if (!(criteria in verifiedSpecs[key])) {
                            verifiedSpecs[key][criteria] = true;
                        }
                    }
                } catch (err) {
                    _didIteratorError2 = true;
                    _iteratorError2 = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion2 && _iterator2['return']) {
                            _iterator2['return']();
                        }
                    } finally {
                        if (_didIteratorError2) {
                            throw _iteratorError2;
                        }
                    }
                }
            }
        }
    }, {
        key: 'wasSpecVerified',
        value: function wasSpecVerified(storyId, criteriaId) {
            var verifiedSpecs = this.verifiedSpecs;

            if (storyId in verifiedSpecs && criteriaId in verifiedSpecs[storyId]) {
                return true;
            } else {
                return false;
            }
        }
    }, {
        key: 'didSpecFail',
        value: function didSpecFail(storyId, criteriaId) {
            var failedSpecs = this.failedSpecs;

            if (storyId in failedSpecs && criteriaId in failedSpecs[storyId]) {

                return true;
            } else {
                return false;
            }
        }
    }, {
        key: 'isAnyTestRunning',
        value: function isAnyTestRunning(allure) {
            return allure.getCurrentSuite() && allure.getCurrentTest();
        }
    }, {
        key: 'dumpJSON',
        value: function dumpJSON(allure, name, json) {
            allure.addAttachment(name, JSON.stringify(json, null, '    ').split('\\"').join('\''), 'application/json');
        }
    }]);

    return AllureReporter;
})(_events2['default'].EventEmitter);

exports['default'] = AllureReporter;
module.exports = exports['default'];
